/* Tree search enabling functions. */

/* The tree search is used by Pierre to determine the best move to play out of the 
 * current board position. This is essentially the heart of the program, as most
 * of the other code is essentially there to make these routines more efficient.
 * The procedure for now is as follows: Search all possible moves up to a depth
 * of 3 ply, then extend the search on capture moves (checking and promoting
 * moves will be added very soon) up to the additional depth specified by the 
 * "qdepth" command line argument. Evaluations are performed only on "leaves"
 * (when the search will not continue deeper). 
 * The basic algorithm for performing the tree search is minimax. Alpha-Beta
 * pruning has been added which essentially recognizes lines which will have
 * no effect on the overall score of a move and therefore do not need to be
 * searched.
 * Functions are:
 *   think()  begins the search,.. will eventually handle the aspiration and
 *            iteritive deepening.
 *   search()  performs the alpha-beta minimax search.
 *   extendSearch()  Handles the quiescence search (currently disabled until
 *                   bugs can be worked out).
 */


/* Returns the amount of time to be spent searching based on the game's time
 * controls and clock times. Returns milliseconds.
 */
long targetTime (void)
  {
  long target;
  int currentMove;

  /* first, set the base time */
  if (game.numMoves != 0)
    {
    currentMove = ((pos.movenum-2)/2+1) % (game.numMoves+1);
    target = game.myTime/(long)(game.numMoves-currentMove+1);
    }
  else
    {
    target = game.gameInc;
    if (checkEndgame(pos.who_moves)) target += game.myTime / (long)15;
    else target += game.myTime / (long)25;
    }
  
  return target;
  }

/* This will perform a search on ONLY capturing moves. Returns a score just
 * like searchlayer. 
 */
int extendSearch (int qdepth, int colour, int bestmax, int bestmin)
  {
  struct MoveList moves;        /* the list of moves out of this position */
  int score;
  int bestscore;
  struct Move tempmove;

  if (pos.who_moves != colour)
    bestscore = 10001;
  else
    bestscore = -10001;

  /* we need to check for repetition here because if it isn't a leaf, then the evaluation
   * function won't pick it up.
   */
  if (checkRepetition(&replist)>=3)
    return -5000;  

  initList(&moves);
  generateMoves(&moves, 1);
  extragenerated+=moves.size;

  if (moves.size == 0)
    { 
    score = evaluate(colour);  
    if (score==10000)
      score = score-50*(DEPTH+QDEPTH-qdepth);
    else if (score==-10000)
      score = score+50*(DEPTH+QDEPTH-qdepth);
    return score;
    }
      
  while (moves.size != 0)
    {
    nummoves++;
    /* get the next move and make it */
    moveCopy(getMove(moves), &tempmove);
    makeMove(&tempmove);
    
    if (qdepth<=1)     /* last ply, evaluate and adjust for possible horizon effects */
      {
      score = evaluate(colour);
     if (tempmove.piece != EMPTY)
       if (check_for_attack(tempmove.to_row, tempmove.to_column, !pos.board[tempmove.to_row][tempmove.to_column].colour))
	 {
	 if (colour == pos.who_moves)
	   score -= pcsq[pos.who_moves][pos.board[tempmove.to_row][tempmove.to_column].piece-1][tempmove.to_row][tempmove.to_column];
	 else
	   score += pcsq[pos.who_moves][pos.board[tempmove.to_row][tempmove.to_column].piece-1][tempmove.to_row][tempmove.to_column];
	 }
      }
    else
      score = extendSearch(qdepth-1, colour, bestmax, bestmin);

    /* If the score is a mate,.. we modify the value so that the mate at the
     * earliest depth recieves the highest score.
     */
    if (score == 10000)
      score = score - 50*(DEPTH+QDEPTH-qdepth);
    else if (score==-10000)
      score = score + 50*(DEPTH+QDEPTH-qdepth);
    
    unmakeMove(tempmove);
    
    if ((score > bestscore) && (pos.who_moves == colour))
      bestscore = score;
    else if ((score < bestscore) && (pos.who_moves != colour))
      bestscore = score;
    deleteMove(&moves);

    /* We still apply alpha-beta here */
    if ((pos.who_moves!=colour) && (score < bestmax))
      { clearList(&moves); return bestscore; }
    if ((pos.who_moves==colour) && (score > bestmin))
      { clearList(&moves); return bestscore; }

    /* now we take care of the modification of the alpha-beta flags */
    if ((pos.who_moves != colour) && (score < bestmin))
      bestmin = score;
    else if ((pos.who_moves == colour) && (score > bestmax))
      bestmax = score;
    }
  return bestscore;
  }


int adjust(struct Move move, int colour, int score)
  {
  if (score<8000 && score>-8000) /* don't adjust mate scores,.. no captures can take place afterwards */
    if (move.piece != EMPTY)
      if (check_for_attack(move.to_row, move.to_column, !pos.board[move.to_row][move.to_column].colour))
        {
        if (colour == !pos.who_moves)
	  score -= pcsq[pos.who_moves][pos.board[move.to_row][move.to_column].piece-1][move.to_row][move.to_column];
        else
	  score += pcsq[pos.who_moves][pos.board[move.to_row][move.to_column].piece-1][move.to_row][move.to_column];
        }
  return score;
  }

void printScore(int score)
  {
  if (score>8000)
    {
    printf("+mat%02d",((10000-score)/100)+1);
    if (FENTEST)
      fprintf(fentestfile, "+mat%02d",((10000-score)/100)+1);
    }
  else if (score<-8000)
    {
    printf("-mat%02d",(10000+score+50)/100);
    if (FENTEST)
      fprintf(fentestfile, "-mat%02d",(10000+score+50)/100);
    }
  else
    {
    printf("%+6.2f",(float)(score-initialEval)/100);
    if (FENTEST)
      fprintf(fentestfile, "%+6.2f",(float)(score-initialEval)/100);
    }
  }

void analysisOut(int score)
  {
  long ms = getMS()-searchStartTime;

  if (XBOARD)
    {
    printf("%4d %6d %10ld %10ld      ",DEPTH,score-initialEval,ms/(long)10,nummoves);
    printPV(pv[0]);
    fflush(stdout);
    return;
    }
  if (FENTEST)
    fprintf(fentestfile, "D:%02d  ", DEPTH);
  outTime(ms);
  printf("  %02d/%02d  ",currentMoveNum,numPossibleMoves);
  if (FENTEST)
     fprintf(fentestfile, "  %02d/%02d  ",currentMoveNum,numPossibleMoves);
  printScore(score); printf("  ");  
  if (FENTEST)
    fprintf(fentestfile, "  ");
  printPV(pv[0]);
  printf("    ");
  }

void aspLowOut(void)
  {
  long ms = getMS()-searchStartTime;

  if (XBOARD)
    return;

  if (FENTEST)
    fprintf(fentestfile, "D:%02d  ", DEPTH);
  outTime(ms);
  printf("  %02d/%02d  ",currentMoveNum,numPossibleMoves);
  printf("    --  ");
  if (FENTEST)
    fprintf(fentestfile, "  %02d/%02d      --  ",currentMoveNum, numPossibleMoves);
  printPV(realPV);
  printf("    ");
  }

void aspHighOut(struct Move mov, int score)
  {
  long ms = getMS()-searchStartTime;
  char output[15];

  createAlgebraOutput(mov, output);
  strcat(output, " <H>");

  if (XBOARD)
    {
    printf("%4d %6d %10ld %10ld      ",DEPTH,score-initialEval,ms/(long)10,nummoves);
    if (pos.movenum%2==0) printf("%d. ... %s\n",pos.movenum/2, output);
    else printf("%d. %s\n",pos.movenum/2+1,output);
    fflush(stdout);
    return;
    }

  if (FENTEST)
    fprintf(fentestfile, "D:%02d  ", DEPTH);
  outTime(ms);
  printf("  %02d/%02d  ",currentMoveNum, numPossibleMoves);
  printf("    ++  ");
  if (FENTEST)
     fprintf(fentestfile, "  %02d/%02d      ++  ",currentMoveNum, numPossibleMoves);
  if (pos.movenum%2==0)
    {
    printf("%d. ... %s\n    ",pos.movenum/2, output);
    if (FENTEST) fprintf(fentestfile, "%d. ... %s\n",pos.movenum/2,output);
    }
  else
    {
    printf("%d. %s\n    ",pos.movenum/2+1, output);
    if (FENTEST) fprintf(fentestfile, "%d. %s\n",pos.movenum/2+1, output);
    }
  }

void failHighOut(struct Move mov, int score)
  {
  long ms = getMS()-searchStartTime;
  char output[10];
  
  unmakeMove(mov);
  createAlgebraOutput(mov, output);
  strcat(output, "!");

  if (XBOARD)
    {
    printf("%4d %6d %10ld %10ld      ",DEPTH,score-initialEval,ms/(long)10,nummoves);
    if (pos.movenum%2==0) printf("%d. ... %s\n",pos.movenum/2, output);
    else printf("%d. %s\n",pos.movenum/2+1,output);
    fflush(stdout);
    makeMove(&mov);
    return;
    }

  if (FENTEST) 
    fprintf(fentestfile, "D:%02d  ", DEPTH);
  outTime(ms);
  printf("  %02d/%02d  ",currentMoveNum, numPossibleMoves);
  printf("    ++  ");
  if (FENTEST) fprintf(fentestfile, "  %02d/%02d      ++  ",currentMoveNum, numPossibleMoves);
  if (pos.movenum%2==0)
    {
    printf("%d. ... %s\n    ",pos.movenum/2, output);
    if (FENTEST) fprintf(fentestfile, "%d. ... %s\n",pos.movenum/2, output);
    }
  else
    {
    printf("%d. %s\n    ",pos.movenum/2+1, output);
    if (FENTEST) fprintf(fentestfile, "%d. %s\n",pos.movenum/2+1, output);
    }
  makeMove(&mov);
  return;
  }

/* search performs the recursive alpha/beta search on the move tree up to the 
 * given depth. Once this depth is exhausted, is calls quiesce() to continue
 * to search only the set of interesting moves. Note that the number of positions
 * evaluated and thus the time consumed by the search increases exponentially
 * with depth.
 * Input Values:
 *  depth:   how many more plies need to be searched.
 * colour:   which side we are trying to find a move for. which side is currently
 *           to move is stored globally in the position structure.
 * bestmin:  the alpha value. Best move for opponent found so far.
 * bestmax:  the beta value. Best move for us found so far.
 * bestMove: will be filled with the best move to play.
 * Output Values:
 *    The score recieved by the best move is returned (in centipawns)
 */
int search(int depth, int colour, int bestmax, int bestmin, struct Move *bestMove)
  {
  struct MoveList moves;        /* the list of moves out of this position */
  struct Move tempmove;         
  int score, bestscore;
  struct Move pvMove;
  int newbest = FALSE;
  int tester=FALSE;

  
  /* At this point if the side to move is the side we're looking for a move for, then we
   * want to find the best move. If it's the opponents turn, we want to find the worse move,
   * because the worse move for us is the best for them, and we assume that is the move
   * which will be played. Scores range from -10000 (mated) to 10000 (mate).
   */ 

  if (pos.who_moves != colour) bestscore = 10001;   
  else bestscore = -10001;

  /* Next we check for draw by repetition. This needs to be checked at every level because
   * the repetition process only checks to see if the current position has been repeated and
   * therefore if it is only checked at eval time (at leaves of the tree), it won't be 
   * detected properly.
   */

  if (depth != 1)
    if (checkRepetition(&replist)>=3)
      return -5000;


  /* Now we generate all pseudolegal moves out of the current position. If there are no
   * moves, the position must be either mated or stalemated,.. and so we evaluate the 
   * position to find out which and return that score. The score is adjusted so that 
   * the mate at the lowest depth is the highest. If there are leftovers from a 
   * previous aspiration search which failed high, we use those instead because it is
   * only necessary to search the remaining moves.
   */

  if (leftovers.size)
    {
    listCopy(leftovers, &moves);
    clearList(&leftovers);
    }
  else
    {
    initList(&moves);
    genMoves(&moves, 0);
    numgenerated+=moves.size;
    }

  if (moves.size == 0)
    {
    score = evaluate(colour);  
    if (score==10000)
      score = score-50*(DEPTH-depth-1);
    else if (score==-10000)
      score = score+50*(DEPTH-depth-1);
    return score;
    }

  /* Before we begin searching them,.. we sort them. Currently this only includes PV
   * moves from a previous lower depth search but will include other kinds of
   * heuristics in the future.
   */
 
  if (lastPV.size)
    {
    sortList(getMove(lastPV), &moves);
    deleteMove(&lastPV);
    }
    
  /* Now we cycle through all the moves and evaluate them. If this isn't the last ply, 
   * the score of a move is based on the score of all it's successors. Otherwise, the
   * score is determined by a positional evaluation, or by the quiescence search.
   */
  while (moves.size != 0)
    {
    nummoves++;
    moveCopy(getMove(moves), &tempmove);
    makeMove(&tempmove);

    /* We need to check if the move is valid, because that has been
     * postponed during the move generation routine.           
     */
    if (check_for_check(!pos.who_moves))
      {
      unmakeMove(tempmove);
      deleteMove(&moves);
      }
    else
      {
      /* The move is valid, so we search it... */

      if (depth==DEPTH) currentMoveNum++;

      if (depth == 1)
        {
        if ((tempmove.piece != EMPTY)&&(QDEPTH>0))
          {
          score=extendSearch(QDEPTH, colour, bestmax, bestmin);
          extralines++;
          }
        else      /* last ply, use evaluate and adjust for possible horizon effects */
          {
          score = adjust(tempmove, colour, evaluate(colour));
	  }
        }

      /* Here I've implemented the minimal window PV search. The PV line is searched first. If this
       * is some other move, we call the search on it with a zero-width window centered around the
       * current best score. If this search fail's low then we have saved time because the true score
       * must be <= to this score and we can ignore it. If however the search fail's high,... we re
       * search it with the normal alpha-beta window.
       */

      else  /* not at last ply,... search successors */
	{
        if (DEPTH==depth && newbest)
          {
          score = search(depth-1, colour,bestscore,bestscore+1, bestMove);
          if (score>=bestscore+1)
            {
	    /* fail high, research */ 
	    /* We know this will become the new best move, so we make it now
             * incase the time runs out 
             */
            if (!TIMEUP)
	      {
              moveCopy(tempmove,bestMove);
              moveCopy(tempmove,&pvMove);
              bestscore=score;
	      if (analysis||showthinking) 
                failHighOut(tempmove, score);
   
              score = search(depth-1, colour, score-1, bestmin, bestMove);
	      }
	    }
	  }
        else  
          score = search(depth-1, colour, bestmax, bestmin, bestMove);
	}

      unmakeMove(tempmove);
      deleteMove(&moves);

      /* Finally,... we now have a score for the move. We adjust the score for nearness
       * in the event of a mate. If the move is the best found so far,... we adjust bestscore
       * to reflect this and copy the move into bestMove. Then we take care of the alpha-beta
       * pruning. We have 2 variables which were passed to us: bestmin and bestmax. They hold
       * the best opponent and side to move scores respectively. If we are searching the
       * opponents moves here, we know we are looking for the minimum score and so the move
       * which will eventually chosen will have a score AT LEAST as low as this one. So, if this
       * score is not high enough to take effect in the rest of the tree (if it is less than
       * best max), we don't have to worry about evaluating the rest of the moves because we
       * are sure that they will have no effect. Parallel logic works for the case where we
       * are searching for the side to move. 
       */

      /* Check to see if time is up. We check here because we don't want this score to have 
       * any effect on the move chosen, because it's search was not completed and thus is
       * inaccurate.
       */
      if (TIMEUP)
        {
        clearList(&moves);
        return bestscore;
	}   

      if (score == 10000)  score = score - 50*(DEPTH-depth);
      else if (score==-10000) score = score + 50*(DEPTH-depth);
    
      if ((score > bestscore) && (pos.who_moves == colour))
        {
        bestscore = score;
        moveCopy(tempmove, &pvMove);
        if (depth==DEPTH)
          {      
          moveCopy(tempmove, bestMove);
          if (score>8000)
	    {
            if (analysis||showthinking)
              {
              storePV(tempmove, depth);
              analysisOut(bestscore);
	      } 
            return bestscore;   /* mate, no need to search further */
	    }
	  }
	}
      else if ((score < bestscore) && (pos.who_moves != colour)) 
        {        
        bestscore = score;
        moveCopy(tempmove, &pvMove);
	}
	
      if ((pos.who_moves!=colour) && (score <= bestmax)) 
        { 
        clearList(&moves); 
        return bestscore; 
        }
      if ((pos.who_moves==colour) && (score >= bestmin)) 
        {
        if (depth==DEPTH)
	  {
	    /* This is an aspiration fail high */
          clearList(&leftovers);
          listCopy(moves, &leftovers);
          addMove(tempmove, &leftovers);
          if (analysis||showthinking)
            aspHighOut(tempmove, bestscore);     
	  }
        clearList(&moves); 
        return bestscore;
        }

      /* We now take care of the modification of the alpha-beta flags.
       * If we have found a move which is the best score found so far 
       * (by each side individually) then it becomes the new alpha or
       * beta value.
       */
      if ((pos.who_moves != colour) && (score < bestmin)) {bestmin = score; newbest=TRUE;}
      else if ((pos.who_moves == colour) && (score > bestmax)) 
        {
        bestmax = score; 
        newbest=TRUE;
        if (depth==DEPTH && (analysis||showthinking))
          {
          storePV(tempmove, depth);
          analysisOut(bestscore);
	  }
        }
      }
    }

  /* It's possible that this position is mated or stalemated and that we
   * haven't actually evaluated any moves because all of the moves generated
   * were invalid. We need to check this case so that we don't return an illegal
   * move.       
   */
  if ((bestscore==10001)||(bestscore==-10001))
    {
    score = evaluate(colour);  
    if (score==10000) score=score-50*(DEPTH-depth-1);
    else if (score==-10000) score = score+50*(DEPTH-depth-1);
    clearList(&(pv[DEPTH-depth]));
    return score;
    }
 
  if (newbest)
    storePV(pvMove, depth);

  /* Are we about to fail low? */
  if (bestscore<bestmax && (analysis||showthinking) && depth==DEPTH)
    aspLowOut();

  return bestscore;
  }

/* think takes care of the entire process of choosing a move to play. It checks the book
 * and/or calls search(), whichever is necessary. The move to be played is returned. If 
 * the game is finished then an illegal move will be returned with from_row=-1.
 */
struct Move think(struct MoveList gameHistory)
  {
  int i;
  int depth = DEPTH;
  int bestmin = 10001;
  int bestmax = -10001;
  struct Move bestMove, tempMove;
  int bestScore, tempScore;
  struct BookMoveList bookMoves;
  int bm;                   /* which book move? */
  struct BookMove booky;
  char output[10];

  /* Set up the stats variables */

  searchStartTime = getMS(); timeGenerating = 0; timeEvaluating = 0;
  time1=time2=time3=time4=0; nummoves = 0; numgenerated = 0;
  numevaluated = 0; extragenerated = 0; extralines = 0;
  initialEval = evaluate(pos.who_moves);


  /* Initialize bestMove and tempMove*/

  bestMove.from_row=-1; bestMove.from_column=-1; bestMove.to_row=-1;
  bestMove.to_column=-1; bestMove.piece=EMPTY; bestMove.promote=EMPTY;
  tempMove.from_row=-1; tempMove.from_column=-1; tempMove.to_row=-1;
  tempMove.to_column=-1; tempMove.piece=EMPTY; tempMove.promote=EMPTY;

  /* Initialize PV lists */

  clearList(&lastPV); clearList(&realPV);
  
  /* First we look for book moves if the book is still enabled. The book is disabled
   * as soon as one search can't find a move in the book. All the possible book moves
   * are chosen from randomly.
   */

  if (book&&!analysis)
    {
    initBookList(&bookMoves);     /* initialize the book moves list */
    bookSearch(gameHistory, &bookMoves);
    if (bookMoves.size == 0)
      book = FALSE;
    else
      {
      /* there are book moves,... choose randomly */
      bm = (int)(((float)rand()/(float)RAND_MAX)*(float)(bookMoves.size-1));
      printf("Book: Choosing %d/%d\n",bm+1, bookMoves.size);
      for (i=0;i<bm;i++)
        deleteBookMove(&bookMoves);
      bookMoveCopy(getBookMove(bookMoves),&booky);
      clearBookList(&bookMoves);
      moveCopy(booky.move, &bestMove);
      }
    } 

  /* If we are not in book mode, or there were no book moves, then we perform the search. 
   * First we check to see if the PCSQ Tables need to be adjusted for the king in the 
   * endgame.
   */

  if (!book||analysis)
    {
    if (!endgame)  /* rearrage the (King) PCSQ tables for the endgame */
      if(checkForEndgame())
	{
	endgame = TRUE;
        setupKingPCSQ(winning=whosWinning());
        }
      else     /* if the other side is winning now,... */
	{
        if(whosWinning() != winning)
	setupKingPCSQ(winning=whosWinning());
	}

    if ((analysis||showthinking)&&!XBOARD)
      {
      printf("Dp  Time         Nodes       #       Score  Principal Variation\n");
      printf("------------------------------------------------------------------------------\n");
      } 

    /* Kick off the iterivite deepening aspiration search */
    TIMEUP=FALSE;
    numPossibleMoves = count_valid_moves();
    DEPTH=0;    /* Starting depth - 1 */
    while(!TIMEUP)
      {
      currentMoveNum = 0;
      DEPTH++;
      if((pv = (struct MoveList *)malloc(DEPTH*sizeof(struct MoveList)))==NULL)
        {
        printf("*** Error! Unable to allocate memory for PV table ***\n");
        fflush(stdout);
        exit(1);
        }
      for (i=0;i<DEPTH;i++)
        initList(&(pv[i]));
      tempMove.from_row=-1;
      if ((analysis||showthinking)&&!XBOARD) {printf("%02d  ",DEPTH); fflush(stdout);}
      tempScore = search(DEPTH, pos.who_moves, bestmax, bestmin, &tempMove);
      tempMove.piece = pos.board[tempMove.to_row][tempMove.to_column].piece;
      if ((pos.board[tempMove.from_row][tempMove.from_column].piece == PAWN) && (tempMove.from_column != tempMove.to_column) && (tempMove.piece == EMPTY))
        tempMove.piece = ENPASSENTPAWN;
      /* check for a failing aspiration search */
      if (!(tempScore > bestmax && tempScore < bestmin))
        {
	/* we need to recall the search */
        for (i=0;i<DEPTH;i++)
          clearList(&(pv[i]));
        clearList(&lastPV);
        listCopy(realPV, &lastPV);    /* get the PV back,  for move ordering */
        if (tempScore >= bestScore+EPSILON)       /* fail high */
          {
	  moveCopy(tempMove,&bestMove);    /* Incase Time runs out during research */
          bestScore=tempScore;
          createAlgebraOutput(tempMove,output);
          //printf("Fail High Move is %02d/%02d [%s]\n",currentMoveNum,numPossibleMoves,output);
          tempMove.from_row=-1;
          currentMoveNum--;
          tempScore = search(DEPTH, pos.who_moves, tempScore-1, 10001, &tempMove);
	  }
        else       /* fail low */
	  {
          currentMoveNum=0;
          tempScore = search(DEPTH, pos.who_moves, -10001, tempScore+1, &tempMove);
	  }
        tempMove.piece = pos.board[tempMove.to_row][tempMove.to_column].piece;
        if ((pos.board[tempMove.from_row][tempMove.from_column].piece == PAWN) && (tempMove.from_column != tempMove.to_column) && (tempMove.piece == EMPTY))
          tempMove.piece = ENPASSENTPAWN;
	}
      if (!XBOARD) printf("\b\b\b\b");
      /* It's possible that time ran out before a best move could be found */
      if (tempScore != 10001 && tempScore != -10001 && tempMove.from_row != -1)
	{
        moveCopy(tempMove, &bestMove);
        bestScore=tempScore;
        bestmax=bestScore-EPSILON;
        bestmin=bestScore+EPSILON;
        listCopy(pv[0],&lastPV);  /* Backup the previous PV for the PVS */
        listCopy(pv[0],&realPV);
    	}
      else
        DEPTH--;     /* no input from this depth */
        
      free(pv);
      if (bestScore>8000)
        break;      /* mate, no need to search deeper */
      else if (bestScore<-8000)
        break;      /* mated, won't find a better score */
      }
    }

  theScore=bestScore-initialEval;
  /* Now we take care of outputing statistics,.. if required. */
  searchEndTime=getMS();
  if (debug||searchstats) printSearchStats();   
  if (whisper) printWhisper(1, bestScore);
  
  if (analysis)
    {
    if (bestMove.from_row==-1)
      printf("This game is finished. It cannot be searched further. (0 possible moves)\n");
    else
      {
      createAlgebraOutput(bestMove, output);
      if (bestScore-initialEval>=0)
        printf("Best move is [%s] which scores +%.2f\n",output,(float)(bestScore-initialEval)/100);
      else
        printf("Best move is [%s] which scores %.2f\n",output,(float)(bestScore-initialEval)/100);
      }
    }

  DEPTH=depth;
  return bestMove;
  }


void testAB (void)
  {
  struct Move tempMove, bestMove;
  int tempScore, bestScore, i;
  char output[10];

  TIMEUP=FALSE;
  numPossibleMoves = count_valid_moves();

  currentMoveNum = 0; 
  if((pv = (struct MoveList *)malloc(DEPTH*sizeof(struct MoveList)))==NULL)
    {
    printf("*** Error! Unable to allocate memory for PV table ***\n");
    exit(1);
    }
  for (i=0;i<DEPTH;i++)
    initList(&(pv[i]));
  tempMove.from_row=-1;
  if (analysis) {printf("%02d  ",DEPTH); fflush(stdout);}
    tempScore = search(DEPTH, pos.who_moves, ALPHA, BETA, &tempMove);
  tempMove.piece = pos.board[tempMove.to_row][tempMove.to_column].piece;
  if ((pos.board[tempMove.from_row][tempMove.from_column].piece == PAWN) && (tempMove.from_column != tempMove.to_column) && (tempMove.piece == EMPTY))
    tempMove.piece = ENPASSENTPAWN;
  printf("\b\b\b\b");
  /* It's possible that time ran out before a best move could be found */
  if (tempScore != 10001 && tempScore != -10001 && tempMove.from_row != -1)
    {
    moveCopy(tempMove, &bestMove);
    bestScore=tempScore;
    listCopy(pv[0],&lastPV);  /* Backup the previous PV for the PVS */
    listCopy(pv[0],&realPV);
    }
        
  free(pv);
   
  createAlgebraOutput(bestMove, output);
  printf("A depth %d search with ALPHA=%d BETA=%d returned move [%s] with score %d\n",DEPTH,ALPHA,BETA,output,bestScore);
  ALPHA=-10001;
  BETA=10001;
  }











