/* Pierre - An Experimental Chess Engine 
 * written by Alex Boby (see pierre.c for more details)
 * ----
 * file [data.h]
 * defines all the data structures and global variables
 */

#define WHITE 1
#define BLACK 0
#define TRUE 1
#define FALSE 0
#define EMPTY 0
#define PAWN 1
#define KNIGHT 2
#define BISHOP 3
#define ROOK 4
#define QUEEN 5
#define KING 6
#define QUEEN_EMPTY 7
#define QUEEN_PAWN 8
#define QUEEN_KNIGHT 9
#define QUEEN_BISHOP 10
#define QUEEN_ROOK 11
#define QUEEN_QUEEN 12
#define ENPASSENTPAWN 14
#define TIMEBASED 0
#define DEPTHBASED 1

typedef long long unsigned int bitboard;

typedef struct Square{
  int colour;       /* what colour is the piece? */
  int piece;        /* what kind of piece is it? */
  } Square;         /* if the piece is 0 then there is no */
		    /* piece on that square */

typedef struct Position {
  struct Square board[8][8];
  int who_moves;            /* Who's turn it is!                         */
  int wKingRow;             /* Keeping track of the position of the king */
  int wKingColumn;          /* speeds many processes                     */
  int bKingRow;
  int bKingColumn;
  int wcastle;              /* these are used for evaluation             */
  int bcastle;
  int wKing;                /* these are flags representing special move  */
  int bKing;                /* possibilities. The piece flags will store  */
  int wqRook;               /* 0 if the piece hasn't moved or the move #  */
  int wkRook;               /* that they moved on if they have moved      */
  int bqRook;               /* (necessary to undo moves properly)         */
  int bkRook;               /* the pawn flags will store the move # which */
  int wPawn[8];             /* each pawn moved forward 2 squares, or 0 if */
  int bPawn[8];             /* they never did.                            */
  int movenum;              /* the movenum is the # of the current move   */
			    /* which is about to be made                  */
  int ippawn;               /* Which column a pawn is ready to be killed  */
                            /* via enpassant, or -1 if none.              */
  } Position;

typedef struct Stat {
  int num_black_pieces;     /* this stucture holds all the stats */
  int num_white_pieces;     /* used for position analysis        */
  int black_strength;
  int white_strength;
  int num_black_moves;      /* Number of valid moves each player */
  int num_white_moves;      /* has to choose from                */
  int white_in_check;       /* boolean: whether or not the indicated  */
  int black_in_check;       /* colour is in check                     */
  } Stat;

typedef struct Game {
  char event[80];
  char site[80];
  int month,day,year,hour,minutes;
  int round;
  char white[80];
  char black[80];
  char result[100];
  int whiteELO;
  int blackELO;
  long gameTime, gameInc;   /* Time controls */
  int numMoves;             /* eg. x moves in y minutes */
  long myTime, oppTime;     /* time remaining */
  } Game;


typedef struct Move {            /* a move's info */
  int from_row;
  int from_column;
  int to_row;
  int to_column;
  int piece;                /* the captured piece (if one) */
  int promote;              /* piece to promote to (if one) */
  } Move;

/* this is the book structure to be used for file i/o */
typedef struct BookMove {
  struct Move move;
  long me;        /* file location of THIS node */
  long next;      /* the next move option */
  long up;        /* the next level from this move */
  int value;
  } BookMove;

typedef struct BookMoveListNode {
  struct BookMove bmov;
  struct BookMoveListNode *next;
  } BookMoveListNode;

typedef struct BookMoveList {
  int size;
  struct BookMoveListNode *head;
  struct BookMoveListNode *tail;
  } BookMoveList;

typedef struct MoveListNode {
  struct Move move;
  struct MoveListNode *next;
  } MoveListNode;

typedef struct MoveList {
  int size;
  struct MoveListNode *head;
  struct MoveListNode *tail;
  } MoveList;

typedef struct Checksum {   /* the checksum holds a position's data in */
    char data[32];          /* as little space as possible             */  
    struct Checksum *next; 
} Checksum;

typedef struct ChecksumList {
  int size;
  struct Checksum *head;
} ChecksumList;



/* These are the Global Variables */
/* ------------------------------ */

// Piece-Square table
int pcsq[2][6][8][8];

/* The bitboard declarations follow */
/* dynamic boards */
bitboard position[6][2];          /* one for each piece and each colour */
bitboard allpieces[2];            /* where all the pieces are together, for each colour */
/* static boards */
bitboard mask[64];                /* only the indexed square is true    */
bitboard doublePawnMask[2];       /* Doubled pawn move mask for each colour */
bitboard leftMask, rightMask;     /* Only the left or right column is false */
bitboard rowmask;                 /* only the first row is true             */
bitboard knightAttacks[64];       /* the squares attacked by a knight on the indexed square */
bitboard kingAttacks[64];         /* the squares attacked by a king on the indexed square */
bitboard slideFriendAttacks[8][256];   /* attack boards for straight sliding pieces (rook, queen) */
bitboard slideEnemyAttacks[8][256];    /* 1. Position of piece in row 2. State of pieces in row */
bitboard friendlyUp[64][256];          /* attack board for diagonal sliding pieces (bishop, queen) */
bitboard friendlyDown[64][256];        /* 1. Position on board 2. State of pieces along diagonal */
bitboard enemyUp[64][256];
bitboard enemyDown[64][256];


struct Stat stats;
struct Position pos;
struct Game game;      /* Game data for PGN header tags */

char *ProgramName = "Pierre v1.7";


int DEPTH = 4;
int QDEPTH = 0;     /* disabled April 16/99 */
int EPSILON = 50;
int ALPHA = -10001;
int BETA = 10001;
long int TIME = 0;   /* Time to spend searching. 0=Auto/Unlimited */
int draw = TRUE;     /* whether or not to draw the board in ascii */
int debug = FALSE;    /* whether or not to display thought info */
int whisper = FALSE;
int XBOARD=FALSE;     /* xboard style output? */
int LOGGAMES=FALSE;     /* write moves to PGN after each game? */
long nummoves = 0;    /* used for evaluating program speed      */
long numgenerated = 0;
long extragenerated = 0;
long numevaluated = 0;
long extralines = 0;
clock_t searchStartTime, searchEndTime;
double timeGenerating = 0;
double timeEvaluating = 0;
double time1, time2, time3, time4 = 0;
int error = 0;        /* Why a move is not valid */
int ep = 0;
int book = TRUE;      /* are we using the book?    */
int theScore = 0;
struct ChecksumList replist;    /* checksum list,.. will be made local l8r */
int CONSOLELOG = FALSE;         /* whether or not console logging is enabled */
char bookfile[255] = "openings.pbf";       /* default name for bookfile */
char pgnlogfile[255] = "gamelog.pgn";      /* file to log games to */
int BOOKMODE = TRUE;   
int piecescores[2];     /* Piece-Square table scores for both colours */
int searchstats = FALSE;        /* whether or not to print search stats after every search */
int winning = WHITE;
int endgame = FALSE;      /* are we at the endgame yet? */
int analysis=FALSE;
int showthinking=FALSE;
int initialEval;
struct MoveList *pv;
struct MoveList lastPV, realPV, leftovers;
int SEARCHMETHOD = TIMEBASED;
int TIMEUP = FALSE;
FILE *fentestfile;
int FENTEST = FALSE;
int numPossibleMoves, currentMoveNum;
char posChecksum[32];








