/* -----------------------------------------------------------------------
 * Pierre V1.7       -------  Chess Engine  -------  Written by Alex Boby
 * -----------------------------------------------------------------------
 * Pierre is an experimental chess engine. He remains in an early 
 * development stage as of now. Begun on March 18th, 1998. All major
 * changes since this date are listed in point form below under 
 * "Additions". All planned additions and work in progress are listed
 * under "To Do". Comments and Suggestions can be emailed to 
 * alex.boby@utoronto.ca.
 * -----------------------------------------------------------------------
 * Additions: Significant Changes in average rating of the program on
 *            ICS's are noted in the form [chessnet, FICS].
 * -----------------------------------------------------------------------
 * April 21st, 1998.    - Major modifications to variable structures made
 *                        to ease future additions. Move history now saved
 *                        for up to 50 moves each side.
 * April 23rd, 1998.    - Castling & en passent are now valid moves.
 * April 28th, 1998.    - Move Lists implemented and History Converted so
 *                        there is no longer a limited # of moves
 * April 28th, 1998.    - Implemented the MiniMax Algorithm
 * May 1st, 1998.       - Implemented the Alpha-Beta algorithm
 * May 15th, 1998.      - Implemented Move Undo's and changed the position
 *                        back to a global structure for speed reasons.
 * May 16th, 1998.      - Interfaced the Program with xboard/winboard
 * June 8th, 1998.      - Fixed various bugs and made minor
 *                        performance enhancements v1.0.
 * June 26th, 1998.     - Basic Opening Book Completed v1.2
 *                        [1450, 1300]
 * October 25th, 1998.  - Better endgame king positioning. ICS Status 
 *                        whispers. v1.3
 * November 23rd, 1998. - Draw by repetition is now considered. v1.4
 * January 14th, 1999.  - Implemented Underpromotion 
 * January 16th, 1999.  - Algebraic Notation is accepted as input.
 * January 17th, 1999.  - Phased move generation added.
 * January 17th, 1999.  - Dynamic piece-square scoring added. 
 * January 21st, 1999.  - Partial quiescence search implemented. v1.5
 *                        [version did not play on ICS due to bugs]
 * April 15th, 1999.    - Completed the conversion to bitboard move
 *                        generation. Also made minor speed enhancements
 *                        such as king tracking.
 * April 18th, 1999.    - Changed the value of the queen in the PCSQ 
 *                        tables from 8 to 9. It was in error.
 *                      - Added an analysis command,... which scores
 *                        every possible move and gives it's best return.
 * April 20th, 1999.    - "Look for Check" algorithm was rewritten and
 *                        program speed increased slightly.
 * April 21st, 1999.    - FEN positional notation is accepted as input
 *                        for starting positions or to run tests from 
 *                        files.
 * April 23rd, 1999.    - Pierre can now output Algebraic Notation.
 *                      - Implemented the xboard 'hint' command.
 * April 24th, 1999.    - Full PGN Support added. Reading, Writing and 
 *                        building the opening book from them are all
 *                        now supported.
 * June 2nd, 1999.      - Search now returns a whole PV instead of just
 *                        the move to be made.
 * June 3rd, 1999.      - Implemented iteritive deepening with the 
 *                        previous PV being searched first.
 *                      - Converted from depth based to time based 
 *                        searching. Updated the analysis function to
 *                        reflect this change.
 * June 5th, 1999.      - Completed the zero-width PV search for inner
 *                        calls after the Principle Variation is searched.
 *                      - Implemented Aspiration Search. Window size 
 *                        defaults to 1/2 pawn but can be set in centi-
 *                        pawns from the command line.
 * June 22nd, 1999.     - Switched from using clock() to using
 *                        gettimeofday() for internal timings. It is now
 *                        accurate!
 * June 26th, 1999.     - Modified the analysis function to support 
 *                        Xboard's "show thinking" option. Also added
 *                        support for the "book" menu item.
 * June 27th, 1999.     - Fixed pgn logging. Date, time, time controls
 *                        are now logged and log file can be specified
 *                        on the command line.
 * July 22nd, 1999.     - Added some king safety into the evaluation.
 * July 24th, 1999.     - Switched to dynamic position checksums. Also
 *                        fixed a bug in repetition detection. It was 
 *                        actually looking for 4 reps because it wasn't
 *                        including the first occurence. v1.7
 *                        [1675, ?].
 * -----------------------------------------------------------------------
 * To Do: (planned immediately indicated by next version # to the left)
 * -----------------------------------------------------------------------
 *                1.9   - Book learning function.
 *                2.0   - Opening names in Book.
 *                1.8   - Hash Tables.
 *                1.8   - Heuristics for move ordering
 *                1.7   - Better Evaluation Function.
 *                2.0   - Save positions in FEN format. 
 *                1.7   - Implement the 50 move rule. (!!!)
 *                2.0   - A working quiescence search!
 *                2.0   - Recognize not enough material draw.                                        
 * -----------------------------------------------------------------------
 * -----------------------------------------------------------------------
 */

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <signal.h>
#include <sys/time.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>

#include "data.h"
#include "pcsq.h"
#include "headers.h"
#include "draw.c"
#include "chess.c"
#include "generate.c"
#include "comp.c"
#include "data.c"
#include "eval.c"
#include "inout.c"
#include "lists.c"
#include "stats.c"
#include "validity.c"
#include "book.c"
#include "bitboard.c"
#include "fen.c"
#include "pgn.c"


/* The main loop ---> TIME TO PLAY CHESS!!! */
int main(int argc, char *argv[])
  {
  int i;
  int temp;
  long temptime;
  char output[10];
  char inputMove[40];
  int computer = BLACK;     /* 2 for 2 player game, otherwise stores computers colour */
  struct Move mov;          /* the pending move */
  struct MoveList history;  /* the game's move history */
  char fenFile[100];
  char pgnFile[100];
  int FENPOS=FALSE;
  int READPGN=FALSE;
  int BUILDPGN=FALSE;
  int numMoves;
  char oppName[60]="Unknown";
  struct tm *currentDate;
  time_t currentTime;
  int stime;
  long ltime;
  FILE *cfp = NULL;


 

  /* First we have to process the Command Line Arguments */
  /* we start at one because the first argument is the program's name */
  for (i=1; i<argc; i++)
    {
    if (!strcmp(argv[i], "depth"))
      {
      if (i<argc-1)
	{
	DEPTH = atoi(argv[i+1]);
	i++;
        SEARCHMETHOD=DEPTHBASED;
	if ((DEPTH > 30) || (DEPTH < 1))
	  {
	  printf("Given depth is not inbounds (1-30)\n");
	  exit(1);
	  }
	}
      }
    if (!strcmp(argv[i],"searchtime"))
      {
      if (i<argc-1)
	{
        TIME = atoi(argv[i+1]);
        i++;
        if (SEARCHMETHOD==DEPTHBASED)
	  {
          printf("Cannot specify both depth & time modes on command line\n");
          exit(1);
	  }
        else if (TIME<0)
          {
          printf("Given search time is invalid (>0)\n");
          exit(1);
          }
        else if (TIME==0)
          printf("Search time set to auto/infinite\n");
        else 
          printf("Search time set to %ldms\n",TIME);
	}
      }  
    else if (!strcmp(argv[i], "epsilon"))
      {
      if (i<argc-1)
	{
	EPSILON = atoi(argv[i+1]);
	i++;
	printf("Epsilon set to %d\n",EPSILON);
	}
      }
    else if (!strcmp(argv[i], "qdepth"))
      {
      if (i<argc-1)
	{
        QDEPTH= atoi(argv[i+1]);
        i++;
        printf("Quiescence depth set to %d\n",QDEPTH);
	}
      }
    else if (!strcmp(argv[i], "bookfile"))
      {
      strcpy(bookfile, argv[i+1]);
      i++;
      }
    else if (!strcmp(argv[i], "fenfile"))
      {
      strcpy(fenFile, argv[i+1]);
      FENPOS=TRUE;
      i++;
      }
    else if (!strcmp(argv[i], "fentest"))
      {
      strcpy(fenFile, argv[i+1]);
      FENTEST=TRUE;
      i++;
      }
    else if (!strcmp(argv[i], "readpgn"))
      {
      strcpy(pgnFile, argv[i+1]);
      i++;
      READPGN=TRUE;
      }
    else if (!strcmp(argv[i], "buildpgn"))
      {
      strcpy(pgnFile, argv[i+1]);
      i++;
      numMoves=atoi(argv[i+1]);
      i++;
      BUILDPGN=TRUE;
      }
    else if (!strcmp(argv[i], "whisper"))
      {
      whisper = TRUE;
      printf("Whisper Enabled\n");
      }
    else if (!strcmp(argv[i], "showthink"))
      showthinking=TRUE;
    else if (!strcmp(argv[i], "nobook"))
      {
      BOOKMODE = FALSE;
      book = FALSE;
      printf("Book mode disabled\n");
      }
    else if (!strcmp(argv[i], "draw"))
      {
      draw = TRUE;
      printf("DRAW mode ON\n");
      }
    else if (!strcmp(argv[i], "debug"))
      {
      debug = TRUE;
      printf("DEBUG mode ON\n");
      }
    else if (!strcmp(argv[i], "loggames"))
      {
      LOGGAMES=TRUE;
      if (i<argc-1)
	{
	strcpy(pgnlogfile,argv[i+1]);
	i++;
	printf("Game logging on. Dump file is %s\n",pgnlogfile);
	}     
      else
	{
        printf("*** You must supply a filename for game logging\n"); 
        exit(1);
	}
      }
    else if (!strcmp(argv[i], "stats"))
      searchstats = TRUE;
    else if (!strcmp(argv[i], "makebook"))
      {
      initBookFile();
      printf("The book file was created.\n");
      exit(1);
      }
    else if (!strcmp(argv[i], "help"))
      {
      printHelpArguments();
      exit(1);
      }
    else if (!strcmp(argv[i], "logconsole"))
      {
      CONSOLELOG=TRUE;
      if(consoleFile(&cfp))
        printf("Console logging is Enabled\n");
      else
        {
        CONSOLELOG=FALSE;
        printf("Console logging is Disabled (Unable to open file)\n");
        }
      }
    else
      {
      printf("[%s] is an Unknown Parameter\n", argv[i]);
      printHelpArguments();
      exit(1);
      }
    }

  
  strcpy(inputMove, "");
  signal (SIGINT, SIG_IGN);     /* ignore the SIGINT's that xboard sends */
  initBoard();                  /* set up the board position */
  initBitboards();              /* set up the bitboards */
  initPCSQ();                   /* set up the piece-square tables */
  initPiecescores();            /* set up the dynamic piecescores */
  initList(&history);           /* initialize the game history list */
  initList(&lastPV);            /* initialize the PV list */
  initList(&realPV);
  initChecksumList(&replist);   /* initialize the checksum list */
  initGame();                   /* initialize the PGN headers */
  positionChecksum();           /* init the checksum string */
  collectStats();

  ltime = time(NULL);           /* seed random # generator */
  stime = (unsigned int) ltime/2;
  srand(stime);

  /* test the book file */
  if (BOOKMODE && !testBook())
    {
    book = FALSE;
    BOOKMODE = FALSE;
    printf("Book mode disabled - file [%s] is not readable\n", bookfile);
    }
 
  printf("%s\n",ProgramName);
  fflush(stdout);
   
  if (READPGN)
    {
    readPGN(pgnFile);
    exit(1);
    }

  if (BUILDPGN)
    {
    buildPGNbook(pgnFile, numMoves);
    exit(1);
    }

  /* Check for FEN position setup */
  if (FENPOS)
    {
    handleFEN(fenFile);
    computer=2;
    BOOKMODE=FALSE;
    }

  if (FENTEST)
    {
    FENTest(fenFile);
    printf("Test Finished.\n");
    exit(1);
    }
 
  if (draw) drawBoard();

  /* Main Driver Loop */
  while (1)
    {
    if (debug) printf("compboards returns %d\n",compareBoards());
    if(checkEndgame(pos.who_moves)) computer = 2;

    /* get input */
    scanf("%s",inputMove);
    logConsole(inputMove,0,&cfp);

    if (!strcmp(inputMove, "go")) {}
    else if (!strcmp(inputMove, "xboard")) {XBOARD=TRUE; draw=FALSE;}
    else if (!strcmp(inputMove, "white")) 
      {
      computer=WHITE; 
      strcpy(game.black, oppName);
      strcpy(game.white, ProgramName);
      }
    else if (!strcmp(inputMove, "black")) 
      {
      computer=BLACK; 
      strcpy(game.white, oppName);
      strcpy(game.black, ProgramName);
      } 
    else if (!strcmp(inputMove, "force")) computer = 2;     /* computer is not playing */
    else if (!strcmp(inputMove, "quit")) break;
    else if (!strcmp(inputMove, "exit")) break;
    else if ((!strcmp(inputMove, "help"))||(!strcmp(inputMove, "?"))) printHelp();    
    else if (!strcmp(inputMove, "helpargs")) printHelpArguments();
    else if (!strcmp(inputMove, "helpcoms")) printHelpCommands();
    else if (!strcmp(inputMove, "showbit")) showBitboards();
    else if (!strcmp(inputMove, "savegame")) logPGN(history);
    else if (!strcmp(inputMove, "depth"))
      {
      scanf("%d",&temp);
      if (temp<=0 || temp>=10)
        printf("** A depth of %d is invalid [1-10]. Depth remains %d\n",temp,DEPTH);
      else
        {
        DEPTH=temp;
        printf("Depth set to %d\n",DEPTH);
	}
      }
    else if (!strcmp(inputMove, "searchtime"))
      {
      scanf("%d",&temp);
      if (temp<0)
	printf("** Search time must be a positive value\n");
      else
        {
        TIME=temp;
        printf("Search time set to %ld\n",TIME);
	}
      }
    else if (!strcmp(inputMove, "name"))
      {
      scanf("%s", oppName);
      if (computer==BLACK||computer==2)
        strcpy(game.white, oppName);
      else
        strcpy(game.black, oppName);
      printf("tellics say %s.\n",ProgramName);
      fflush(stdout);
      }
    else if (!strcmp(inputMove, "rating"))
      {
      if (computer==BLACK||computer==2)
        {
        scanf("%d",&game.blackELO);
        scanf("%d",&game.whiteELO);
	}
      else
        {
        scanf("%d",&game.whiteELO);
        scanf("%d",&game.blackELO);
	}
      }
    else if (!strcmp(inputMove, "level"))
      {
      scanf("%d",&game.numMoves);      /* x moves */
      scanf("%ld",&game.gameTime);
      game.gameTime *= 60000;          /* convert from minutes to milliseconds */
      game.myTime = game.gameTime;
      game.oppTime = game.gameTime;
      scanf("%ld",&game.gameInc);
      game.gameInc *= 1000;            /* convert from seconds to milliseconds */
      }
    else if (!strcmp(inputMove, "book"))
      printf("%d\n",book);
    else if (!strcmp(inputMove, "testbit"))
      testBitboard(0);
    else if (!strcmp(inputMove, "testbitc"))
      testBitboard(1);
    else if (!strcmp(inputMove, "testgen"))
      testGeneration();
    else if (!strcmp(inputMove, "compboards"))
      printf("returns %d\n",compareBoards());
    else if (!strcmp(inputMove, "compbit"))
      compBitboards();
    else if (!strcmp(inputMove, "draw"))
      {
      printf("tellics say Sorry,... I don't yet know how to consider draw requests!\n");
      fflush(stdout);
      }
    else if (!strcmp(inputMove, "new"))
      {
      computer = BLACK;         /* begin a new game */
      initBoard();
      initBitboardPositions();
      initPiecescores();
      clearList(&history);
      initList(&history);
      clearChecksumList(&replist);
      initChecksumList(&replist);
      positionChecksum();
      collectStats();
      currentTime = time(NULL);
      currentDate = localtime(&currentTime);
      game.month = currentDate->tm_mon;
      game.day = currentDate->tm_mday;
      game.year = currentDate->tm_year;
      game.hour = currentDate->tm_hour;
      game.minutes = currentDate->tm_min;
      strcpy(game.black, ProgramName);
      if (BOOKMODE) book=TRUE;
      endgame = FALSE;
      initPCSQ();               /* <-- it could have been altered by endgame */
      if (draw) drawBoard();
      }
    else if (!strcmp(inputMove, "beep")) {}
    else if (!strcmp(inputMove, "random")) {}
    else if (!strcmp(inputMove, "hard")) {}
    else if (!strcmp(inputMove, "post"))
      showthinking=TRUE;
    else if (!strcmp(inputMove, "nopost"))
      showthinking=FALSE;
    else if (!strcmp(inputMove, "time")) 
      {
      scanf("%ld",&game.myTime);
      game.myTime *= 10;     /* given in hundredths of seconds, need milliseconds */
      }
    else if (!strcmp(inputMove, "otim")) 
      {
      scanf("%ld",&game.oppTime);
      game.oppTime *= 10;
      }
    else if (!strcmp(inputMove, "drawboard")) drawBoard();
    else if (!strcmp(inputMove, "rep")) printf("Position repeated %d times\n",checkRepetition(&replist));
    else if (!strcmp(inputMove, "hint"))
      {
      moveCopy(think(history), &mov);
      createAlgebraOutput(mov, output);
      printf("Hint: %s\n",output);
      fflush(stdout);
      }
    else if (!strcmp(inputMove, "testalg"))
      {
      scanf("%s",inputMove);
      moveCopy(parseAlgebra(inputMove), &mov);
      if (mov.from_row == -1)
        printf("Invalid Algebraic Move\n");
      else if (mov.from_row == -2)
        printf("Ambiguous Algebraic Move\n");
      else
        {
        createMoveOutput(mov, output);
        printf("Corresponds to move %s\n",output);
        }
      }
    else if (!strcmp(inputMove, "result"))
      {
      gets(game.result);
      strcpy (game.result, &game.result[1]);
      if (LOGGAMES) logPGN(history);
      }    
    else if (!strcmp(inputMove, "debugflags"))
      {
      printf("White Pawns: ");
      for (i=0;i<8;i++)
	    printf("%d ", pos.wPawn[i]);
      printf("\nBlack Pawns: ");
      for (i=0;i<8;i++)
	    printf("%d ", pos.bPawn[i]);
      printf("\nippawn is %d\n", pos.ippawn);
      printf("Black King is on [%d,%d]\n",pos.bKingRow,pos.bKingColumn);
      printf("White King is on [%d,%d]\n",pos.wKingRow,pos.wKingColumn);
      }
    else if (!strcmp(inputMove, "showbook")||!strcmp(inputMove, "bk"))
      showBook(history);
    else if (!strcmp(inputMove, "updatebook"))
      {
      if (BOOKMODE)
        printf("%d moves added to book\n",bookUpdate(history));
      else
        printf("* You must have the book enabled to update it's contents\n");
      }
    else if (!strcmp(inputMove, "ab"))
      {
      scanf("%d",&ALPHA);
      scanf("%d",&BETA);
      temp=analysis;
      analysis=TRUE;
      testAB();
      analysis=temp;
      }
    else if (!strcmp(inputMove, "analysis"))
      {
      temp=analysis;
      analysis=TRUE;
      think(history);
      analysis=temp;
      }
    else if (!strcmp(inputMove, "undo"))
      {
      if (history.size)
	{
        moveCopy (getMove(history), &mov);
        unmakeMove(mov);
        deleteMove(&history);
	}
      else
        printf("There are no moves to unmake!\n");
      }
    else                       /* It must be a move by the user */
      {
      moveCopy(parseMoveInput(inputMove), &mov);
      if (mov.from_row == -1)
        moveCopy(parseAlgebra(inputMove), &mov);
      if (mov.from_row == -1)
        {
        printf("Illegal move (%s) [no matching move]\n", inputMove);
        fflush(stdout);
        }
      else if (mov.from_row == -2)
        {
        printf("Illegal move (%s) [ambiguous move]\n", inputMove);
        fflush(stdout);
	}
      else        /* it's a move! */
	{
	mov.piece = pos.board[mov.to_row][mov.to_column].piece;
	if ((pos.board[mov.from_row][mov.from_column].piece == PAWN) && (mov.from_column != mov.to_column) && (mov.piece == EMPTY))
	   mov.piece = ENPASSENTPAWN;
	if (good_move(mov))
	  {
	  makeMove(&mov);
	  addMove(mov, &history);
	  collectStats();
	  if (draw) drawBoard();
	  }
	else
	  {
	  if (debug)
	    printf("[move failed on condition %d]\n",error);
	  printf("Illegal move (%s) [no matching move]\n", inputMove);
	  fflush(stdout);
	  }
	}
      }
    if (pos.who_moves == computer)
      {
      if (!XBOARD && !TIME)
        {
        printf("You must set a target time when not playing with clocks. Use searchtime [x milliseconds]\n");
        }
      else
        {
        if (!checkEndgame(pos.who_moves))
	  {
          temptime = TIME;
          if (TIME==0)
            TIME = targetTime();
          moveCopy(think(history),&mov);
          TIME=temptime;
          if (XBOARD)
            {
	    createMoveOutput(mov, output);
	    printf("%d. ... %s\n",(pos.movenum+1)/2, output);
            }
          else
            {
            createAlgebraOutput(mov, output);
            if (pos.who_moves==WHITE)
              printf("%d. %s\n",(pos.movenum+1)/2, output);
            else
              printf("%d. ... %s\n",(pos.movenum+1)/2, output);
	    }
          fflush(stdout);
   	
	  makeMove(&mov);
	  addMove(mov, &history);
	  collectStats();
	  if (draw) drawBoard();
          }
	}
      }   
    }
  clearList(&history);
  clearList(&realPV);
  clearList(&lastPV);
  if (cfp!=NULL) fclose(cfp);
  return TRUE;
  }


























